#!/usr/bin/env python3

import sys
from mml2mfvi import *

# use a ROM with expanded instruments but without moved .inst data
# e.g. t-edition instrument patch, or nascentorder with 'i' and without 'b'
ROMFILE = "e:/stuff/beyondchaos/ff5ff6mcs2/mytest.smc"

IDP = 0x4C #patch/enh: Searching for Friends
IDN = 0x17 #nopatch/nat: Narshe
IDM = 0x20 #multipart: Phantom Train
multipart_use_enh = False
also_battles = True
# ----------- no need to edit below this line

ADDRESSP = 0x370000
BYTEADRP = b"\x00\x00\xF7"
ADDRESSN = 0x380000
BYTEADRN = b"\x00\x00\xF8"
ADDRESST = 0x390000
BYTEADRT = b"\x00\x00\xF9"
BYTEADRM = BYTEADRP if multipart_use_enh else BYTEADRN

INSTDATA = 0x53F95
PTRDATA = 0x53E96
INSTP = INSTDATA + IDP*0x20
INSTN = INSTDATA + IDN*0x20
INSTM = INSTDATA + IDM*0x20
INSTT = INSTM
PTRP = PTRDATA + IDP*3
PTRN = PTRDATA + IDN*3
PTRM = PTRDATA + IDM*3
PTRT = PTRM

if len(sys.argv) >= 2:
    MMLFILE = sys.argv[1]
    silent_mode = True
else:
    MMLFILE = input("mml file?\n>")
    silent_mode = False

try:    
    with open(MMLFILE, 'r') as f:
        mml = f.readlines()

    variants = mml_to_akao(mml)
    for k, v in variants.items():
        variants[k] = (bytes(v[0],encoding="latin-1"), bytes(v[1],encoding="latin-1"))
    data = variants["_default_"][0]
    inst = variants["_default_"][1]

    if len(data) < 0x1000:
        data += b"\x00" * (0x1000 - len(data))
        
    with open(ROMFILE, 'r+b') as f:
        f.seek(ADDRESSP)
        f.write(data)
        f.seek(INSTP)
        f.write(inst)
        f.seek(PTRP)
        f.write(BYTEADRP)
        if multipart_use_enh:
            f.seek(INSTM)
            f.write(inst)
            f.seek(PTRM)
            f.write(BYTEADRM)
        if also_battles:
            f.seek(PTRDATA + 0x24 * 3)
            f.write(BYTEADRP)
            f.seek(INSTDATA + 0x24 * 0x20)
            f.write(inst)
        
    if "nopatch" in variants:    
        data = variants["nopatch"][0]
        inst = variants["nopatch"][1]
    elif "nat" in variants:    
        data = variants["nat"][0]
        inst = variants["nat"][1]    
    elif "tier2" in variants:
        data = variants["tier2"][0]
        inst = variants["tier2"][1]

    with open(ROMFILE, 'r+b') as f:
        f.seek(ADDRESSN)
        f.write(data)
        f.seek(INSTN)
        f.write(inst)
        f.seek(PTRN)
        f.write(BYTEADRN)
        if not multipart_use_enh:
            f.seek(INSTM)
            f.write(inst)
            f.seek(PTRM)
            f.write(BYTEADRM)
    
    if "tr" in variants:
        data = variants["tr"][0]
        inst = variants["tr"][1][0:-2] + b"\x0F\x00"
        
        with open(ROMFILE, 'r+b') as f:
            f.seek(ADDRESST)
            f.write(data)
            f.seek(INSTT)
            f.write(inst)
            f.seek(PTRT)
            f.write(BYTEADRT)
        
    if not silent_mode:
        print("done.")
except Exception as e:
    print(e)
    if silent_mode:
        input()

if not silent_mode:
    input()